@@ -1,4 +1,4 @@
-# $Id: mvnorm.R 243 2012-12-06 12:42:34Z thothorn $
+# $Id: mvnorm.R 264 2014-04-04 10:19:18Z thothorn $
 
 rmvnorm<-function (n, mean = rep(0, nrow(sigma)), sigma = diag(length(mean)),
                    method=c("eigen", "svd", "chol"), pre0.9_9994 = FALSE)
@@ -66,11 +66,18 @@ dmvnorm <- function (x, mean, sigma, log=FALSE)
     if (length(mean) != NROW(sigma)) {
         stop("mean and sigma have non-conforming size")
     }
-    distval <- mahalanobis(x, center = mean, cov = sigma)
-    logdet <- sum(log(eigen(sigma, symmetric=TRUE,
-                                   only.values=TRUE)$values))
-    logretval <- -(ncol(x)*log(2*pi) + logdet + distval)/2
+    if( !is.null(dim(mean)) ) dim(mean) <- NULL
+
+    ### <faster code contributed by Matteo Fasiolo mf364 at bath.ac.uk
+    dec <- chol(sigma)
+    tmp <- forwardsolve(t(dec), t(x) - mean)
+    rss <- colSums(tmp ^ 2)
+    logretval <- - sum(log(diag(dec))) - 0.5 * length(mean) * log(2 * pi) - 0.5 * rss
+    ### />
+
+    names(logretval) <- rownames(x)
+
     if(log) return(logretval)
     exp(logretval)
 }
-  
+  
\ No newline at end of file
